package org.processmining.markovprocess.models;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

public class DtmcPreventiveMaintenanceProblem {

	final DiscreteTimeMarkovChain dtmc;
	final double discount;
	final Policy policy;
	final double correctiveCost;
	final double preventiveCost;
	final int numberOfValueIterations = 10000;
	final List<Double> maxDifferences = new ArrayList<Double>();
	
	public DtmcPreventiveMaintenanceProblem(DiscreteTimeMarkovChain dtmc, double discount, double correctiveCost, double preventiveCost) {
		this.discount = discount;
		this.dtmc = dtmc;
		this.correctiveCost = correctiveCost;
		this.preventiveCost = preventiveCost;
		
		policy = new Policy(dtmc.keySet());
		
		for (int i = 0; i < numberOfValueIterations; i++) {
			double maxDifference = 0d;
			for (String state : dtmc.keySet()) {
				double oldValue = policy.get(state).getValue();
				policy.put(state, updatePolicy(state));
				maxDifference = Math.max(maxDifference, Math.abs(oldValue-policy.get(state).getValue()));
			}
			maxDifferences.add(maxDifference);
		}
		
		// Sanity checks:
		for (String state : dtmc.keySet()) {
			if (policy.get(state).getKey()==Action.REPAIR && !dtmc.get(state).containsKey(DiscreteTimeMarkovChain.failedState)) {
				System.out.println("SANITY CHECK FAILED FOR "+state);
			}
		}
		
		System.out.println("DONE");
	}
	
	private Entry<Action,Double> updatePolicy(String state) {
		// Failed state
		if (state.equals(DiscreteTimeMarkovChain.failedState)) {
			double cost = correctiveCost;
			for (Entry<String,Double> entry : dtmc.getInitialDistribution().entrySet()) {
				// No discount because immidiate transition
				cost += entry.getValue()*policy.get(entry.getKey()).getValue();
			}
			return Policy.createEntry(Action.REPAIR,cost);
		}
		Double repairCost = preventiveCost;
		for (Entry<String,Double> entry : dtmc.getInitialDistribution().entrySet()) {
			// Again, no discount
			repairCost += entry.getValue()*policy.get(entry.getKey()).getValue();
		}
		
		Double waitCost = 0d;
		for (Entry<String,Double> entry : dtmc.get(state).entrySet()) {
			waitCost += discount*entry.getValue()*policy.get(entry.getKey()).getValue();
		}

		// Decide
		if (repairCost < waitCost) {
			return Policy.createEntry(Action.REPAIR, repairCost);
		} else {
			return Policy.createEntry(Action.WAIT, waitCost);
		}
	}
	
	public Policy getPolicy() {
		return policy;
	}
}
