package org.processmining.markovprocess.models;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public class Policy implements Map<String,Entry<Action,Double>> {
	private Map<String,Entry<Action,Double>> policy = new HashMap<String,Entry<Action,Double>>();
	private Map<String,Double> dangers = new HashMap<String,Double>();
	
	public Policy(Collection<String> states) {
		for (String state : states)
			policy.put(state, createEntry(null,0d));
	}
	
	public double getDanger(String state) {
		return dangers.get(state);
	}
	
	public void setDangers(ContinuousTimeMarkovChain ctmc, double discount) {
		for (String state : ctmc.keySet()) {
			dangers.put(state, ctmc.getDtmc().getTransitionProbability(state, DiscreteTimeMarkovChain.failedState)/(discount-ctmc.getIntensity(state, state)));
		}
	}
	
	public List<Entry<String,Double>> getSortedFailProbabilities() {
		List<Entry<String,Double>> entryList = new ArrayList<Entry<String,Double>>(dangers.entrySet());
		Collections.sort(entryList, new Comparator<Map.Entry<String,Double>>() {
			public int compare(Entry<String, Double> o1, Entry<String, Double> o2) {
				return o1.getValue().compareTo(o2.getValue());
			}
		});
		return entryList;
	}
	
	public int size() {
		return policy.size();
	}

	public boolean isEmpty() {
		return policy.isEmpty();
	}

	public boolean containsKey(Object key) {
		return policy.containsKey(key);
	}

	public boolean containsValue(Object value) {
		return policy.containsValue(value);
	}

	public java.util.Map.Entry<Action, Double> get(Object key) {
		return policy.get(key);
	}

	public java.util.Map.Entry<Action, Double> put(String key, java.util.Map.Entry<Action, Double> value) {
		return policy.put(key, value);
	}

	public java.util.Map.Entry<Action, Double> remove(Object key) {
		return policy.remove(key);
	}

	public void putAll(Map<? extends String, ? extends java.util.Map.Entry<Action, Double>> m) {
		policy.putAll(m);
	}

	public void clear() {
		policy.clear();
	}

	public Set<String> keySet() {
		return policy.keySet();
	}

	public Collection<java.util.Map.Entry<Action, Double>> values() {
		return policy.values();
	}

	public Set<java.util.Map.Entry<String, java.util.Map.Entry<Action, Double>>> entrySet() {
		return policy.entrySet();
	}

	public static Entry<Action,Double> createEntry(final Action a, final Double c) {
		return new Entry<Action,Double>() {
			@Override
			public Action getKey() {
				return a;
			}

			@Override
			public Double getValue() {
				return c;
			}

			@Override
			public Double setValue(Double value) {
				// Just don't
				return null;
			}
		};
	}
}
