package org.processmining.markovprocess.models;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class StateLifetimes implements Map<String, List<Double>> {
	private final Map<String,List<Double>> lifetimes;
	private final Map<String,Double> averages;
	
	public StateLifetimes(Map<String,List<Double>> lifetimes) {
		this.lifetimes = lifetimes;
		this.averages = new HashMap<>();
		
		for (Entry<String,List<Double>> entry : lifetimes.entrySet()) {
			double total = 0;
			for (Double lifetime : entry.getValue()) {
				total += lifetime;
			}
			
			averages.put(entry.getKey(), total / entry.getValue().size());
		}
	}
	
	public Double getAverage(Object key) {
		return averages.get(key);
	}
	
	@Override
	public int size() {
		return lifetimes.size();
	}

	@Override
	public boolean isEmpty() {
		return lifetimes.isEmpty();
	}

	@Override
	public boolean containsKey(Object key) {
		return lifetimes.containsKey(key);
	}

	@Override
	public boolean containsValue(Object value) {
		return lifetimes.containsValue(value);
	}

	@Override
	public List<Double> get(Object key) {
		return lifetimes.get(key);
	}

	@Override
	public List<Double> put(String key, List<Double> value) {
		return lifetimes.put(key, value);
	}

	@Override
	public List<Double> remove(Object key) {
		return lifetimes.remove(key);
	}

	@Override
	public void putAll(Map<? extends String, ? extends List<Double>> m) {
		lifetimes.putAll(m);
	}

	@Override
	public void clear() {
		lifetimes.clear();
	}

	@Override
	public Set<String> keySet() {
		return lifetimes.keySet();
	}

	@Override
	public Collection<List<Double>> values() {
		return lifetimes.values();
	}

	@Override
	public Set<java.util.Map.Entry<String, List<Double>>> entrySet() {
		return lifetimes.entrySet();
	}
}
