package org.processmining.markovprocess.plugins;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.RoundingMode;
import java.text.DecimalFormat;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.ContinuousTimeMarkovChain;

@Plugin(name = "Continuous Time Markov Chain Exporter",
parameterLabels = { "Continuous Time Markov Chain", "File" },
returnLabels = { },
returnTypes = { },
userAccessible = true)
@UIExportPlugin(description = "Regular txt file (.txt)", extension = "txt")
public class ContinuousTimeMarkovChainExportPlugin {
	private static final int totalSpaces = 8;
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public void export(
			PluginContext context,
			ContinuousTimeMarkovChain ctmc,
            File file) throws IOException {
	    // For rounding
	    DecimalFormat df = new DecimalFormat("#.##");
	    df.setRoundingMode(RoundingMode.HALF_UP);
	    
	    FileWriter writer = new FileWriter(file);
	    PrintWriter pwriter = new PrintWriter(writer);
	    
	    pwriter.print(addSpaces("f\\t"));
	    for (String state : ctmc.getDtmc().getStatesInOrder()) {
	    	pwriter.print(addSpaces(state));
	    }
	    pwriter.println();
	    
	    for (String from : ctmc.getDtmc().getStatesInOrder()) {
	    	pwriter.print(addSpaces(from));
	    	for (String to : ctmc.getDtmc().getStatesInOrder())
	    		pwriter.print(addSpaces(df.format(ctmc.getIntensity(from, to))));
	    	pwriter.println();
	    }
	    pwriter.close();
	}
	
	private String addSpaces(String s) {
		return s+new String(new char[totalSpaces-s.length()]).replace("\0", " ");
	}
}
