
package org.processmining.markovprocess.plugins;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.RoundingMode;
import java.text.DecimalFormat;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.DiscreteTimeMarkovChain;
import org.processmining.markovprocess.models.StateClustering;

@Plugin(name = "Discrete Time Markov Chain Exporter",
parameterLabels = { "Discrete Time Markov Chain", "File", "State Clustering" },
returnLabels = { },
returnTypes = { },
userAccessible = true)
@UIExportPlugin(description = "Regular txt file (.txt)", extension = "txt")
public class DiscreteTimeMarkovChainExportPlugin {
	final static boolean forMatlab = true;
	
	private static final int totalSpaces = 8;
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public void export(
			PluginContext context,
			DiscreteTimeMarkovChain dtmc,
            File file) throws IOException {
	    // For rounding
	    DecimalFormat df = new DecimalFormat("#.#####");
	    df.setRoundingMode(RoundingMode.HALF_UP);
	    
	    FileWriter writer = new FileWriter(file);
	    PrintWriter pwriter = new PrintWriter(writer);
	    
	    if (!forMatlab) {
		    pwriter.print(addSpaces("f\\t"));
		    for (String state : dtmc.getStatesInOrder()) {
		    	pwriter.print(addSpaces(state));
		    }
		    pwriter.println();
	    }
	    
	    for (String from : dtmc.getStatesInOrder()) {
	    	if (!forMatlab) {
	    		pwriter.print(addSpaces(from));
	    	}
	    	for (String to : dtmc.getStatesInOrder())
	    		pwriter.print(addSpaces(df.format(dtmc.getTransitionProbability(from, to))));
	    	pwriter.println();
	    }
	    pwriter.close();
	}

	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0, 1, 2 })
	public void exportClustered(
			PluginContext context,
			DiscreteTimeMarkovChain dtmc,
            File file, StateClustering clustering) throws IOException {
	    // For rounding
	    DecimalFormat df = new DecimalFormat("#.#####");
	    df.setRoundingMode(RoundingMode.HALF_UP);
	    
	    FileWriter writer = new FileWriter(file);
	    PrintWriter pwriter = new PrintWriter(writer);
	    
	    if (!forMatlab) {
		    pwriter.print(addSpaces("f\\t"));
		    for (String state : dtmc.getStatesInOrder()) {
		    	pwriter.print(addSpaces(state));
		    }
		    pwriter.println();
	    }
	    
	    for (String from : clustering.getOrdering()) {
	    	if (!forMatlab) {
	    		pwriter.print(addSpaces(from));
	    	}
	    	for (String to : dtmc.getStatesInOrder())
	    		pwriter.print(addSpaces(df.format(dtmc.getTransitionProbability(from, to))));
	    	pwriter.println();
	    }
	    pwriter.close();
	}
	
	private String addSpaces(String s) {
		return s + (!forMatlab ? new String(new char[totalSpaces-s.length()]).replace("\0", " ") : "\t");
	}
}
