package org.processmining.markovprocess.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.DiscreteTimeMarkovChain;
import org.processmining.markovprocess.models.StateClustering;
import org.processmining.markovprocess.models.StationaryDistribution;

@Plugin(name = "Find Clustering of the DTMC or Stationary Distribution",
parameterLabels = { "Discrete Time Markov Chain", "Stationary Distribution" },
returnLabels = { "State Clustering" },
returnTypes = { StateClustering.class },
userAccessible = true)
public class FindClusteringPlugin {
	/**
	 * The amount of eigenvectors deep the clustering goes.
	 */
	public final static int clusterDepth = 6;
	
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0 })
	public static StateClustering clusterFromDtmc(
			  final PluginContext context,
			  final DiscreteTimeMarkovChain dtmc) {
		return dtmc.getCluster(clusterDepth);
	}

	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 1 })
	public static StateClustering clusterFromStationaryDistribution(
			  final PluginContext context,
			  final StationaryDistribution sd) {
		return new StateClustering(sd, clusterDepth);
	}
}
