package org.processmining.markovprocess.plugins;

import java.util.ArrayList;
import java.util.List;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.ContinuousTimeMarkovChain;
import org.processmining.markovprocess.models.Policy;
import org.processmining.markovprocess.models.StateLifetimes;

@Plugin(name = "Find Jump States Based On Average Time To Live",
parameterLabels = { "Continuous Time Markov Chain" },
returnLabels = { "State Clustering" },
returnTypes = { Policy.class },
userAccessible = true)
public class FindJumpStatesPlugin {
	
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0 })
	public static Policy findJumpStates(
			  final PluginContext context,
			  final ContinuousTimeMarkovChain ctmc) {
		List<String> jumpStates = new ArrayList<>();
			StateLifetimes lifetimes = ctmc.getStateLifetimes();
			for (String s : ctmc.keySet()) {
				boolean isMax = true;
				for (String from : ctmc.getIncomingStates(s)) {
					isMax &= lifetimes.getAverage(from) < lifetimes.getAverage(s);
				}
				if (isMax)
					jumpStates.add(s);
			}
			return null;
	}
}
