package org.processmining.markovprocess.plugins;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Map.Entry;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.InitialDistribution;

@Plugin(name = "Policy Exporter",
parameterLabels = { "InitialDistribution", "File" },
returnLabels = { },
returnTypes = { },
userAccessible = true)
@UIExportPlugin(description = "Regular txt file (.txt)", extension = "txt")
public class InitialDistributionExportPlugin {
	private static final int totalSpaces = 8;
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public void export(
			PluginContext context,
			InitialDistribution initialDistribution,
            File file) throws IOException {
	    // For rounding
	    DecimalFormat df = new DecimalFormat("#.#####");
	    df.setRoundingMode(RoundingMode.HALF_UP);
	    
	    FileWriter writer = new FileWriter(file);
	    PrintWriter pwriter = new PrintWriter(writer);
	    
	    pwriter.println(addSpaces("State")+"Probability");
	    for (Entry<String,Double> entry : initialDistribution.entrySet()) {
	    	pwriter.println(addSpaces(entry.getKey())
	    			+df.format(entry.getValue()));
	    }
	    pwriter.close();
	}
	
	private String addSpaces(String s) {
		return s+new String(new char[totalSpaces-s.length()]).replace("\0", " ");
	}
}

