package org.processmining.markovprocess.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.ContinuousTimeMarkovChain;
import org.processmining.markovprocess.models.CtmcPreventiveMaintenanceProblem;
import org.processmining.markovprocess.models.DiscreteTimeMarkovChain;
import org.processmining.markovprocess.models.DtmcPreventiveMaintenanceProblem;
import org.processmining.markovprocess.models.Policy;

@Plugin(name = "Solve Preventive Maintenance problem",
parameterLabels = { "Discrete Time Markov Chain", "Continuous Time Markov Chain" },
returnLabels = { "Policy" },
returnTypes = { Policy.class },
userAccessible = true)
public class PreventiveMaintenanceProblemPlugin {
	
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0 })
	public static Policy findDtmcPolicy(
			  final PluginContext context,
			  final DiscreteTimeMarkovChain dtmc) {
		return new DtmcPreventiveMaintenanceProblem(dtmc,0.99d, 1000,1).getPolicy();
	}
	
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 1 })
	public static Policy findCtmcPolicy(
			  final PluginContext context,
			  final ContinuousTimeMarkovChain ctmc) {
		return new CtmcPreventiveMaintenanceProblem(ctmc,0.01d, 1000,1).getPolicy();
	}
}
