package org.processmining.markovprocess.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.DiscreteTimeMarkovChain;
import org.processmining.markovprocess.models.StateClustering;

@Plugin(name = "Reorder DTMC states",
parameterLabels = { "Discrete Time Markov Chain", "Ordering" },
returnLabels = { "Reordered DTMC" },
returnTypes = { DiscreteTimeMarkovChain.class },
userAccessible = true)
public class ReorderDtmcPlugin {
	/**
	 * The amount of eigenvectors deep the clustering goes.
	 */
	public final static int clusterDepth = 6;
	
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public static DiscreteTimeMarkovChain cluster(
			  final PluginContext context,
			  final DiscreteTimeMarkovChain dtmc,
			  final StateClustering sc) {
		dtmc.setStatesOrder(sc.getOrdering());
		return dtmc;
	}
}
