package org.processmining.markovprocess.plugins;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.Sign;
import org.processmining.markovprocess.models.StateClustering;

@Plugin(name = "State Clustering Exporter",
parameterLabels = { "State Clustering", "File" },
returnLabels = { },
returnTypes = { },
userAccessible = true)
@UIExportPlugin(description = "Regular txt file (.txt)", extension = "txt")
public class StateClusteringExportPlugin {
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public void export(
			PluginContext context,
			StateClustering stateClustering,
            File file) throws IOException {
	    
	    FileWriter writer = new FileWriter(file);
	    PrintWriter pwriter = new PrintWriter(writer);
	    
	    for (List<Sign> signs : stateClustering.getSignOrdering()) {
	    	pwriter.print(signsToString(signs)+"\t");
	    	for (String state : stateClustering.get(signs)) {
	    		pwriter.print(state+", ");
	    	}
	    	pwriter.println();
	    }
	    pwriter.close();
	}
	
	private String signsToString(List<Sign> signs) {
		String result = "";
		for (Sign sign : signs)
			result += sign;
		return result;
	}
}
