package org.processmining.markovprocess.plugins;

import java.util.List;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.markovprocess.models.StateClustering;
import org.processmining.markovprocess.models.StationaryDistribution;

@Plugin(name = "Cluster States From DTMC Stationary Distribution",
parameterLabels = { "StationaryDistribution" },
returnLabels = { "State Clustered Stationary Distribution", "State Clustering" },
returnTypes = { StationaryDistribution.class, StateClustering.class },
userAccessible = true)
public class StateClusteringPlugin {
	/**
	 * The amount of eigenvectors deep the clustering goes.
	 */
	public final static int clusterDepth = 6;
	
	@UITopiaVariant(affiliation = "TU/e",
            author = "Martijn M. Gösgens",
            email = "m.m.gosgens@student.tue.nl",
            uiLabel = UITopiaVariant.USEPLUGIN)
	@PluginVariant(requiredParameterLabels = { 0 })
	public static Object[] cluster(
			  final PluginContext context,
			  final StationaryDistribution sd) {
		// We're changing the object that is passed, this might result
		// in some reference bugs since the objects are passed by reference.
		StateClustering cluster = new StateClustering(sd,clusterDepth);
		List<String> newOrdering = cluster.getOrdering();
		sd.setOrdering(newOrdering);
		return new Object[] {sd, cluster};
	}
}
